<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Command;

use App\Entity\Core\ExportJob;
use App\Entity\Core\ExportTemplate;
use App\Service\ConfigExportImport\ExportService;
use App\Entity\Core\File;
use App\Entity\Core\PgFile;
use App\EventSubscriber\MqMessagebroadcastSubscriber;
use App\Service\SettingsHelper;
use Doctrine\ORM\EntityManagerInterface;
use Exception;
use Ramsey\Uuid\Uuid;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Class ExportJobCommand
 */
class ExportJobCommand extends Command
{
    const COMPONENT = 'component';
    const ENTITY_TYPE = 'entityType';
    const HTTP_METHOD = 'operation';
    const REMOTE_ADDR = 'source';
    const DELTA_DATA = 'data';
    const TRANSACTION_DATE = 'transactionDate';
    const UUID = 'entityUuid';
    const USER_UUID = 'userId';
    const USER = 'user';

    /** @var Filesystem $filesystem */
    protected $filesystem;

    /** @var SettingsHelper */
    private $settingsHelper;

    private $entityManager;

    /** @var ExportService $exportService */
    private $exportService;

    /** @var ContainerInterface */
    private $container;

    public function __construct(
        SettingsHelper $settingsHelper,
        EntityManagerInterface $entityManager,
        ExportService $exportService,
        ContainerInterface $container
    ) {
        $this->settingsHelper = $settingsHelper;
        $this->entityManager = $entityManager;
        $this->exportService = $exportService;
        $this->container = $container;
        parent::__construct();
    }

    protected function configure()
    {
        $this
            ->setName('fortisoar:jobs:export')
            ->setDescription('Exports records and configuration into a zip file')
            ->setDefinition([
                new InputOption(
                    'job-uuid',
                    'j',
                    InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY,
                    'The uuid of the export job that will be started.'
                ),
                new InputOption(
                    'user-token',
                    't',
                    InputOption::VALUE_OPTIONAL,
                    'The user token for the export context. If empty, the appliance user will be used.'
                ),
                new InputOption(
                    'token-type',
                    'tt',
                    InputOption::VALUE_OPTIONAL,
                    'Type of token HMAC or JWT.'
                )
            ]);

        $this->filesystem = new Filesystem();
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $io = new SymfonyStyle($input, $output);

        $jobUuids = $input->getOption('job-uuid');
        if (count($jobUuids) == 0) {
            $io->error('Please specify a job uuid');
            return;
        }
        $jobUuid = $jobUuids[0];
        $rawToken = $input->getOption('user-token');
        $tokenType = $input->getOption('token-type');
        if ($rawToken && $tokenType) {
            $this->settingsHelper->setRefreshedToken($rawToken, $tokenType);
        }
        $repository = $this->entityManager->getRepository(ExportJob::class);
        /** @var ExportJob $exportJob */
        $exportJob =  $repository->findOneBy(['uuid' => $jobUuid]);

        $status = 'In Progress';
        $this->updateExportJob($exportJob, $status, null, null);

        try {
            $configExportData = $this->exportService->exportConfiguration($exportJob, $io, $tokenType);
        } catch (\Exception $e) {
            $this->updateExportJob($exportJob, 'Error', null, null, $e->getMessage());
            $io->error(sprintf('Error while exporting file: %s', $e->getMessage()));
        }
        if ($configExportData) {
            $jsonData = json_encode($configExportData, JSON_PRETTY_PRINT);
            $filename = $exportJob->getFileName();
            $filename = $filename ? $filename : 'config.json';
            $mimetype = 'application/json';
            $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));

            $pgFile = new PgFile();
            /** @var AbstractFile $file */
            $pgFile->setFile(base64_encode($jsonData));
            $pgFile->setUuid(Uuid::uuid4()->toString());
            $this->entityManager->persist($pgFile);
            $this->entityManager->flush();

            $file = new File();
            $file->setFile("/api/3/pgfile/" . $pgFile->getUuid());
            $file->setFilename($filename);
            $file->setMimeType($mimetype);
            $file->setUploadDate($timestamp);
            $file->setSize(strlen($jsonData));
            $file->setUuid(Uuid::uuid4()->toString());
            $this->entityManager->persist($file);
            $this->entityManager->flush();

            $status = 'Export Complete';
            $this->updateExportJob($exportJob, $status, null, "/api/3/files/" . $file->getUuid());

            $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));
            $exportTemplate = $exportJob->getTemplate();
            $exportTemplate->setLastExportDate($timestamp);
            $this->entityManager->persist($exportTemplate);
            $this->entityManager->flush();

            $io->success(sprintf('Export completed successfully.'));
        }
    }

    /**
     * @return array
     */
    protected function prepareChangeData(array $oldData, array $newData, string $entityId)
    {
        $deltaData = [];
        $deltaData['oldData'] = $oldData;
        $deltaData['newData'] = $newData;
        try {
            /** @var App\Entity\Actors\Actor $user */
            $user = $this->settingsHelper->getCurrentUser();
            if ($user) {
                $userUUID = $user->getUuid();
                $userName = $this->settingsHelper->extractUserName($user);
            } else {
                $userUUID = 'SYSTEM';
                $userName = 'SYSTEM';
            }
        } catch (Exception $e) {
            $userUUID = 'SYSTEM';
            $userName = 'SYSTEM';
        }
        $sourceIp = 'localhost';
        if (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {
            $sourceIp = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } elseif (isset($_SERVER['REMOTE_ADDR'])) {
            $sourceIp = $_SERVER['REMOTE_ADDR'];
        }
        return [
            self::COMPONENT => 'crudhub',
            self::ENTITY_TYPE => 'ExportJob',
            self::UUID => $entityId,
            self::USER_UUID => $userUUID,
            self::USER => $userName,
            self::HTTP_METHOD => 'Update',
            self::TRANSACTION_DATE => (int)($_SERVER['REQUEST_TIME_FLOAT'] * 1000),
            self::DELTA_DATA => $deltaData,
            self::REMOTE_ADDR => $sourceIp
        ];
    }

    private function updateExportJob(ExportJob $exportJob, $status = null, $progressPercent = null, $file = "", $errorMessage = null)
    {
        $oldData = [
            '@id' => '/api/3/export_jobs/' . $exportJob->getUuid()
        ];
        $newData = [
            '@id' => '/api/3/export_jobs/' . $exportJob->getUuid()
        ];
        $oldData['id'] = $exportJob->getId();
        $newData['id'] = $exportJob->getId();
        if ($status !== null) {
            $oldStatus = $exportJob->getStatus();
            $exportJob->setStatus($status);
            $oldData['status'] = $oldStatus;
            $newData['status'] = $status;
        }
        if ($progressPercent !== null) {
            $oldProgressPercent = $exportJob->getProgressPercent();
            $exportJob->setProgressPercent($progressPercent);
            $oldData['progressPercent'] = $oldProgressPercent;
            $newData['progressPercent'] = $progressPercent;
        }
        if ($file !== "") {
            $oldFile = $exportJob->getFile();
            $exportJob->setFile($file);
            $oldData['file'] = $oldFile;
            $newData['file'] = $file;
        }
        if ($errorMessage !== null) {
            $oldErrorMessage = $exportJob->getErrorMessage();
            $exportJob->setErrorMessage($errorMessage);
            $oldData['errorMessage'] = $oldErrorMessage;
            $newData['errorMessage'] = $errorMessage;
        }
        $timestamp = new \DateTime('NOW', new \DateTimeZone('UTC'));
        $exportJob->setModifyDate($timestamp);
        $this->entityManager->persist($exportJob);
        $this->entityManager->flush();
        $body = $this->prepareChangeData($oldData, $newData, $exportJob->getUuid());
        $this->container->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->setContentType('application/json');
        $this->container->get('old_sound_rabbit_mq.cyops_crud_auditing_producer')->publish(json_encode([$body]), MqMessagebroadcastSubscriber::DEFALUT_KEY);
    }
}
