<?php

namespace App\Controller;

use App\Entity\Query\Query;
use App\Providers\QueryProvider;
use App\Security\Voter\FsrVoter;
use Doctrine\Inflector\Inflector;
use App\Constants\AggregateOperators;
use Doctrine\Inflector\InflectorFactory;
use Symfony\Component\HttpFoundation\Request;
use App\Providers\PermissionsDictionaryBuilder;
use App\Providers\AggregateDataProviderInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use ApiPlatform\Core\DataProvider\CollectionDataProviderInterface;
use ApiPlatform\Core\Serializer\SerializerContextBuilderInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use ApiPlatform\Core\Metadata\Resource\Factory\ResourceMetadataFactoryInterface;
use App\Service\MetadataHelper;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class AdvancedQueryController extends BaseController
{
    const DEFAULT_KEYS = ['id', '@id', '@type'];
    const SELECTIVE_FIELD_KEY = '__selectFields';
    protected $security = ["roles", "teams", "import_jobs", "export_templates", "proxies", "module_permissions", "field_permissions"];
    protected $application = [
        "tags", "system_settings", "generic_template", "system_view_templates", "picklists", "picklist_names", "modules",
        "attribute_metadatas", "staging_model_metadatas", "model_metadatas",
    ];
    protected $workflows = ["workflows", "rules", "workflow_collections", "rule_collections"];

    /** @var  QueryProvider */
    private $queryProvider;

    private $collectionDataProvider;

    private $serializerContextBuilder;

    protected $inflector;

    protected $fsrVoter;

    protected $resourceMetadataFactory;
    
    protected $aggregateDataProvider;

    public function __construct(ResourceClassResolverInterface $resourceClassResolver, QueryProvider $queryProvider, 
        CollectionDataProviderInterface $collectionDataProvider, SerializerContextBuilderInterface $serializerContextBuilder, 
        ContainerInterface $containers, FsrVoter $fsrVoter, ResourceMetadataFactoryInterface $resourceMetadataFactory, 
        AggregateDataProviderInterface $aggregateDataProvider, TagAwareCacheInterface $cache, MetadataHelper $metadataHelper)
    {
        $this->queryProvider = $queryProvider;
        $this->collectionDataProvider = $collectionDataProvider;
        $this->serializerContextBuilder = $serializerContextBuilder;
        $this->fsrVoter = $fsrVoter;
        $this->aggregateDataProvider = $aggregateDataProvider;
        $this->resourceMetadataFactory = $resourceMetadataFactory;
        $this->inflector = InflectorFactory::create()->build();
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    public function applyQuery(Request $request, $resourceShortName)
    {
        $perm = $this->getModulePermission($resourceShortName);

        // ToDo remove once for all system entities mmd fixtures are added
        if (!$perm) {
            if (in_array($resourceShortName, $this->security)) {
                $perm = 'security';
            } else if (in_array($resourceShortName, $this->application)) {
                $perm = 'application';
            } else {
                $perm = $resourceShortName;
            }
        }

        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . $perm)) {
            throw new AccessDeniedException();
        }
        $resource = $this->getResourceFromClassName($resourceShortName);
        $resourceMetadata = $this->resourceMetadataFactory->create($resource);
        $content = $request->getContent();
        $query = json_decode($content, true) ?? [];
        $data = $this->evaulateQuery($request, $query, $resource);
        return JsonResponse::create($data);
    }

    public function applyNamedQuery()
    {
    }

    protected function evaulateQuery($request, $query, $resource)
    {

        $this->queryProvider->setInitialQuery($query);
        $queryLimit = $request->get('$limit', 30);
        $queryPage = $request->get('$page', 1);
        $attributes = [];
        $attributes['resource_class'] = $resource;
        $attributes['collection_operation_name'] = "get";
        $attributes['filters'] = ['$limit' => $queryLimit, '$page' => $queryPage];

        $context = $this->serializerContextBuilder->createFromRequest($request, true, $attributes);

        /** @var AggregateDataProviderInterface $aggregateDataProvider */
        $data = $this->isAggregateQuery($query) ?
            $this->aggregateDataProvider->getAggregate($resource, $request, $query) :
            $this->collectionDataProvider->getCollection($resource, $attributes['collection_operation_name'], $attributes);
        $serializer = $this->container->get('serializer');

        $context = $this->serializerContextBuilder->createFromRequest($request, true, $attributes);
        $data1 = $serializer->normalize($data, 'jsonld', $context);
        return $data1;
    }

    protected function isAggregateQuery($query)
    {
        $isAggregateQuery = false;

        if (array_key_exists(Query::KEYWORD_AGGREGATES, $query)) {
            foreach ($query[Query::KEYWORD_AGGREGATES] as $aggregate) {
                if (!in_array($aggregate['operator'], [AggregateOperators::SELECT, AggregateOperators::FIELDS])) {
                    $isAggregateQuery = true;
                    break;
                }
            }
        }

        return $isAggregateQuery;
    }
}
