<?php

namespace App\Controller;

use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use App\Constants\CacheConstants;
use App\Entity\ModelMetadata\ModelMetadata;
use App\Entity\ModelMetadata\StagingModelMetadata;
use App\Service\MetadataHelper;
use Doctrine\Inflector\InflectorFactory;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

abstract class BaseController extends AbstractController
{
    protected $inflector;

    protected $resourceClassResolver;

    protected $containers;

    protected $cache;

    protected $metadataHelper;

    const NORMALIZER_METHOD = 'jsonld';
    const SELECTIVE_FIELDS = '__selectFields';
    const API_PREFIX = '/api/3/';
    const RESOURCE_SECURITY = 'security';

    public function __construct(ResourceClassResolverInterface $resourceClassResolver, ContainerInterface $containers, 
        TagAwareCacheInterface $cache, MetadataHelper $metadataHelper)
    {
        $this->inflector = InflectorFactory::create()->build();
        $this->resourceClassResolver = $resourceClassResolver;
        $this->containers = $containers;
        $this->cache = $cache;
        $this->metadataHelper = $metadataHelper;
    }

    /**
     * @param string $type
     * @return StagingModelMetadata|null
     */
    public function getClassNameByType($type)
    {
        //TODO: get this from metadata helper
        /** @var ManagerRegistry $managerRegistry */
        $managerRegistry = $this->container->get('doctrine');
        $queryManager = $managerRegistry->getManagerForClass(ModelMetadata::class);
        $queryRepository = $queryManager->getRepository(ModelMetadata::class);
        return $queryRepository->findOneBy(array('type' => $type));
    }

    public function getResourceFromClassName($resourceShortName)
    {
        $resource = null;
        $typeClassMap = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        if (!$typeClassMap->isHit()) {
            // load it from the file cache
            $this->metadataHelper->createModuleMapCache();
            $typeClassMap = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        } 
        [$classPath, $module] = $typeClassMap->get()[$resourceShortName];
        if ($this->resourceClassResolver->isResourceClass($classPath)) {
            $resource = $this->resourceClassResolver->getResourceClass(null, $classPath);
        }
        if ($resource === null) {
            $model = $this->getClassNameByType($resourceShortName);
            if ($model === null) {
                throw new BadRequestHttpException(sprintf('Resource "%s" not found.', $resourceShortName));
            }
            $singular = $model->getDescriptions()['singular'];
            $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
            $resourceName = $this->inflector->classify($escapedSingular);
            $classPath = $this->containers->getParameter("type_map")[$resourceName] ?? '\\App\\Entity\\Custom\\' . $escapedSingular;
            $resource = $this->resourceClassResolver->getResourceClass(null, $resourceName);
        }
        if (!$resource) {
            throw new BadRequestHttpException(sprintf('Resource "%s" not found.', $resourceShortName));
        }
        return $resource;
    }

    public function getModulePermission($resourceShortName)
    {
        $typeClassMap = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        if (!$typeClassMap->isHit()) {
            $this->metadataHelper->createModuleMapCache();
            $typeClassMap = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        } 
        [$classPath, $module] = $typeClassMap->get()[$resourceShortName];
        return $module;
    }

    public function getSuccessResponse($data, $resource = null, $status = Response::HTTP_OK)
    {
        $normalized_data = $this->get('serializer')->normalize(
            $data,
            self::NORMALIZER_METHOD,
            ['resource_class' => $resource]
        );
        $response = new JsonResponse(
            $normalized_data,
            $status
        );
        return $response;
    }
}
