<?php

namespace App\Controller;

use stdClass;
use App\Service\MetadataHelper;
use App\Service\UtilityService;
use App\Security\Voter\FsrVoter;
use App\Constants\CacheConstants;
use App\Controller\BaseController;
use App\Service\BulkRequestService;
use Doctrine\Inflector\InflectorFactory;
use Doctrine\ORM\EntityManagerInterface;
use App\Entity\ModelMetadata\ModelMetadata;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Contracts\Cache\TagAwareCacheInterface;
use Symfony\Component\Serializer\SerializerInterface;
use ApiPlatform\Core\Api\ResourceClassResolverInterface;
use App\Entity\Workflow\Workflow;
use App\Entity\Workflow\WorkflowCollection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

class UpsertController extends BaseController
{

    protected $entityManager;
    protected $queryProvider;
    protected $serializerContextBuilder;
    protected $fsrVoter;
    protected $eventDispatcher;
    protected $validator;
    protected $bulkRequestService;
    protected $utilityService;
    protected $serializer;

    public function __construct(
        ResourceClassResolverInterface $resourceClassResolver,
        ContainerInterface $containers,
        FsrVoter $fsrVoter,
        EntityManagerInterface $entityManager,
        MetadataHelper $metadataHelper,
        UtilityService $utilityService,
        BulkRequestService $bulkRequestService,
        TagAwareCacheInterface $cache,
        SerializerInterface $serializer
    ) {
        $this->fsrVoter = $fsrVoter;
        $this->entityManager = $entityManager;
        $this->bulkRequestService = $bulkRequestService;
        $this->inflector = InflectorFactory::create()->build();
        $this->utilityService = $utilityService;
        $this->serializer = $serializer;
        parent::__construct($resourceClassResolver, $containers, $cache, $metadataHelper);
    }

    const DISALLOWED_MODULES = ['tenants', 'routers', 'agents', 'comments', 'actors', 'files', 'images'];

    protected function prepare_filter_data($requestBody, $filterItems)
    {
        $filter = [];
        foreach ($filterItems as $item) {
            if (is_string($item)) {
                $filter[$item] = $requestBody[$item] ?? null;
            } else {
                $filter = array_merge($filter, get_object_vars($item));
            }
        }
        return $filter;
    }

    public function get_unique_resource_fields($request_data, $resourceShortName)
    {
        $permission = $this->getModulePermission($resourceShortName);
        if (isset($request_data->{'__unique'}) && $request_data->{'__unique'}) {
            return [$request_data->{'__unique'}, $permission];
        }
        $cacheName = 'constraintField_' . $resourceShortName;
        $item = $this->cache->getItem($cacheName);
        if ($item->isHit()) {
            return $item->get();
        }
        $managerRegistry = $this->container->get('doctrine');
        $manager = $managerRegistry->getManagerForClass(ModelMetadata::class);
        $modelMetaDataRepository = $manager->getRepository(ModelMetadata::class);
        $modelMetaData = $modelMetaDataRepository->findOneBy(array('type' => $resourceShortName));
        if (!$modelMetaData) {
            return [[], $permission];
        }
        $uniueConstraint = count($modelMetaData->getUniqueConstraint()) ? $modelMetaData->getUniqueConstraint()[0] : [];
        $module = $modelMetaData->getType();
        $columns = [];
        foreach ($uniueConstraint as $key => $value) {
            $columns = isset($value['columns']) ? $value['columns'] : [];
        }
        $item->set([$columns, $module]);
        $item->expiresAfter(3000);
        $item->tag(CacheConstants::UPSERT_MODULE_FIELDS);
        $this->cache->save($item);
        return [$columns, $module];
    }

    public function upsertAction(Request $request, $resourceShortName)
    {
        if (in_array($resourceShortName, self::DISALLOWED_MODULES)) {
            throw new BadRequestHttpException('Upsert Operation for ' . $resourceShortName . ' modules is not supported.');
        }
        $resource = $this->getResourceFromClassName($resourceShortName);
        $request_data = json_decode($request->getContent());

        [$filterItems, $module] = $this->get_unique_resource_fields($request_data, $resourceShortName);

        $replaceIfExist = isset($request_data->{'__replace'}) ? $request_data->{'__replace'} : true;

        $requestBody = $request_data;
        $this->bulkRequestService->updateReferenceToIriAndCheckForRelation($requestBody);

        // $utilsObj = new Utils();
        $objectData = json_decode(json_encode($requestBody), true);

        $data_filters = $this->prepare_filter_data($objectData, $filterItems);
        $object = [];
        if ($data_filters) {
            $entityRepo = $this->entityManager->getRepository($resource);
            $object = $this->utilityService->getItemWithWhere($entityRepo, $data_filters);
        }
        if (count($object) && !$replaceIfExist) {
            return $this->getSuccessResponse($object[0], $resource);
        }
        if (count($object)) {
            // Existing Object Found, trying to update the existing object
            if (!$this->fsrVoter->isGranted("update." . $module)) {
                throw new AccessDeniedException();
            }
            $object = $object[0];
            $this->utilityService->ValidateOwnerShip($request, $object);
            $groups = UtilityService::identifyChangeFields($requestBody);

            // As per change in computation of workflow and other component
            $normalizedPreviousData = $this->serializer->normalize(
                $object,
                'jsonld',
                ["asJson" => true, "relationships" => true, "groups" => $groups]
            );
            $request->attributes->set('fsr_previous_data', $normalizedPreviousData);
            $request->attributes->set('fsr_change_fields', $groups);

            $denormalizedObject = $this->get('serializer')->deserialize(
                $request->getContent(),
                $resource,
                'jsonld',
                ['object_to_populate' => $object, 'fetch_data' => false]
            );
            $request->attributes->set('_api_item_operation_name', 'put');
            $request->attributes->set('previous_data', $object);
        } else {
            // Create the new record
            if (!$this->fsrVoter->isGranted("create." . $module)) {
                throw new AccessDeniedException();
            }
            $denormalizedObject = $this->serializer->deserialize(
                $request->getContent(),
                $resource,
                'jsonld',
                ['fetch_data' => false]
            );
            $request->attributes->set('_api_collection_operation_name', 'post');
            $request->attributes->set('_api_item_operation_name', 'post');
        }
        $request->attributes->set('_api_resource_class', $resource);
        $request->attributes->set('data', $denormalizedObject);
        $request->attributes->set('_format', 'jsonld');
        $request->overrideGlobals();
        return $denormalizedObject;
    }

    public function bulkUpsertAction(Request $request, $resourceShortName, $objectData = null)
    {
        $resourceClass = $this->getResourceFromClassName($resourceShortName);
        $objectDataClone = new stdClass();
        $headers = $request->headers->all();
        $queryParams = $request->query->all();

        if (is_null($objectData)) {
            $objectData = json_decode($request->getContent());
            if (array_key_exists(self::SELECTIVE_FIELDS, $objectData)) {
                $selectedFields = $objectData->{self::SELECTIVE_FIELDS};
                $queryParams[self::SELECTIVE_FIELDS] = implode(',', $selectedFields);
            }
            if (array_key_exists("data", $objectData)) {
                $objectDataClone = $objectData;
            } else if (array_key_exists("__data", $objectData)) {
                $objectDataClone->{'data'} = $objectData->{'__data'};
            } else {
                $objectDataClone->{'data'} = $objectData;
            }

            if ((array_key_exists("__unique", $objectData))) {
                foreach ($objectDataClone->{'data'} as $data) {
                    $data->{'__unique'} = $objectData->{'__unique'};
                }
            }
        } else {
            // $objectDataClone->{'__unique'} = ['uuid'];
            foreach ($objectData as $data) {
                $data->{'__unique'} = ['uuid'];
            }
            $objectDataClone->{'data'} = $objectData;
        }

        return $this->bulkRequestService->prepareRecordSubRequest(
            $resourceClass,
            $objectDataClone,
            '/api/3/upsert/' . $resourceShortName,
            $headers,
            $queryParams
        );
    }

    public function bulkUpsertWorkflowAction(Request $request)
    {
        $request_data = json_decode($request->getContent());
        $requestDatas = property_exists($request_data, "__data") ? $request_data->__data : $request_data;
        $objectDatas = json_decode(json_encode($requestDatas), true);
        $objectDatas = $this->upsertWorkflows($objectDatas);
        return $this->bulkUpsertAction($request, 'workflows', json_decode(json_encode($objectDatas)));
    }

    public function bulkUpsertWorkflowCollectionsAction(Request $request)
    {
        $request_data = json_decode($request->getContent());
        $requestDatas = property_exists($request_data, "__data") ? $request_data->__data : $request_data;
        $objectDatas = json_decode(json_encode($requestDatas), true);
        $objectDatas = $this->upsertWorkflowCollections($objectDatas);
        return $this->bulkUpsertAction($request, 'workflow_collections', json_decode(json_encode($objectDatas)));
    }

    public function modify_workflow_steps_routes($existingItems, $newItems, $prefix)
    {
        foreach ($newItems as $index => $newitem) {
            foreach ($existingItems as $existingItem) {
                if ($existingItem->getUuid() == $newitem['uuid']) {
                    $newItems[$index]['@id'] = $prefix . $newitem['uuid'];
                    break;
                }
            }
        }
        return $newItems;
    }

    public function upsertWorkflowCollections($workflowCollections)
    {
        foreach ($workflowCollections as $index => $workflowCollection) {
            $workflows = $workflowCollection['workflows'];
            $workflowCollections[$index]['workflows'] = $this->upsertWorkflows($workflows);
        }
        return $workflowCollections;
    }


    public function upsertWorkflows($workflows)
    {
        $resource = Workflow::class;
        foreach ($workflows as $index => $workflow) {
            $data_filters = $this->prepare_filter_data($workflow, ['uuid']);
            $workflowRepository = $this->entityManager->getRepository(Workflow::class);
            $object = $this->utilityService->getItemWithWhere($workflowRepository, $data_filters);
            if (count($object)) {
                $workflow['@id'] = '/api/3/workflows/' . $workflow['uuid'];
                $workflow['steps'] =  $this->modify_workflow_steps_routes($object[0]->getSteps(), $workflow['steps'], '/api/3/workflow_steps/');
                $workflow['routes'] = $this->modify_workflow_steps_routes($object[0]->getRoutes(), $workflow['routes'], '/api/3/workflow_routes/');
            }
            $workflows[$index] = $workflow;
        }
        return $workflows;
    }
}
