<?php

namespace App\EventSubscriber;

use App\Traits\Ownable;
use App\Event\DataEvent;
use App\Traits\Trackable;
use App\Entity\Custom\Tenant;
use App\Traits\PeerReplicable;
use App\Constants\CacheConstants;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ViewEvent;
use ApiPlatform\Core\Util\RequestAttributesExtractor;
use Symfony\Component\HttpKernel\Exception\HttpException;
use ApiPlatform\Core\DataPersister\DataPersisterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class MsspEventSubscriber implements EventSubscriberInterface
{

    protected $containers;

    protected $tokenStorage;

    protected $entityManager;

    protected $requeststack;

    protected $cache;

    private const MASTER_ROLE_NAME = 'master';

    private const IS_DISTRIBUTED_INSTANCE = 'isDistributedInstance';

    private const IS_INSTANCE_ROLE = 'instanceRole';

    public function __construct(ContainerInterface $containers, TokenStorageInterface $tokenStorage, EntityManagerInterface $entityManager, 
        DataPersisterInterface $dataPersister, TagAwareCacheInterface $cache)
    {
        $this->containers = $containers;
        $this->tokenStorage = $tokenStorage;
        $this->entityManager = $entityManager;
        $this->dataPersister = $dataPersister;
        $this->cache = $cache;
    }

    public static function getSubscribedEvents()
    {
        return [
            KernelEvents::VIEW => ['handleObjectForMsspChanges', 34],
            DataEvent::NAME => ['onDataEvent', 34],
        ];
    }

    public function onDataEvent(DataEvent $event)
    {
        $request = $event->getRequest();
        $object = $event->getControllerResult();
        $previous_data = $event->getPreviousObject();
        $this->msspOperations($request, $object, $previous_data);
    }

    public function handleObjectForMsspChanges(ViewEvent $event)
    {
        // UnComment return statement line if not testing mssp
        // Until cache is not set
        // return;

        $request = $event->getRequest();
        $object = $event->getControllerResult();
        $previous_data = $event->getRequest()->attributes->get('previous_data');

        if (
            $object instanceof Response
            || $request->isMethodSafe()
            || !($attributes = RequestAttributesExtractor::extractAttributes($request))
            || !$attributes['persist']
        ) {
            return;
        }

        if (!$this->dataPersister->supports($object, $attributes)) {
            return;
        }

        $this->msspOperations($request, $object, $previous_data);
    }

    protected function msspOperations($request, $object, $previous_data, $event = null)
    {
        $isDistributedInstance = $this->checkForMsspInstance();
        if (!$isDistributedInstance) {
            return;
        }
        if (is_null($previous_data)) {
            // POST Request
            $this->setTenantObject($object);
        } else {
            // PUT REQUEST
            $request_body = json_decode($request->getContent());
            $conflict = $this->isConflcts($object, $previous_data, $request_body);
            if ($conflict) {
                $previous_data->setConflict($conflict);
                if ($event) {
                    $event->setControllerResult($previous_data);
                }
            }
        }
    }

    public function isConflcts($object, $previous_data, $request_body = null)
    {
        if (isset($request_body->{'__replicateData'})) {
            return false;
        }
        $entityClass = get_class($object);
        if ($this->isPeerReplicable($entityClass) && $this->isTrackable($entityClass)) {
            $instanceRole = $this->getInstanceRole();
            $modifyUserUuidOfRequestedObj = $this->tokenStorage->getToken()->getUser()->getUuid();
            $modifyUserUuidOfExistingObj = $previous_data->getModifyUser()->getUuid();
            $modifyDateOfExistingObj = $previous_data->getModifyDate()->getTimestamp() ?? 0;
            $modifyDateOfRequestedObj = $request_body->{'modifyDate'} ?? PHP_INT_MAX;
            if ($instanceRole != self::MASTER_ROLE_NAME && ($modifyUserUuidOfRequestedObj != $modifyUserUuidOfExistingObj) && ($modifyDateOfExistingObj >= $modifyDateOfRequestedObj)) {
                return true;
            }
            return false;
        }
    }

    /**
     * @return ProxyClient
     */
    public function getDasClient()
    {
        return $this->containers->get("app.proxy.client.das");
    }

    public function isPeerReplicable($entity_class)
    {
        return in_array(PeerReplicable::class, class_uses($entity_class));
    }

    public function isTrackable($entity_class)
    {
        return in_array(Trackable::class, class_uses($entity_class));
    }

    public function isOwnable($entity_class)
    {
        return in_array(Ownable::class, class_uses($entity_class));
    }

    public function getDetailsFromDas()
    {
        $dasUri = $this->containers->getParameter('das.uri');
        $dasClient = $this->getDasClient();
        $headers = array(
            "Content-type" => "application/json",
        );
        $instanceRole = self::MASTER_ROLE_NAME;
        $isDistributedInstance = false;
        $dasRequests = new GuzzleRequest('get', $dasUri . '/license/?param=license_details', $headers);
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $dasResponseData = $dasResponse->getBody()->getContents();
            $jsonDecodeResponse = json_decode($dasResponseData, true);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode < 300) {
                $isDistributedInstance = $jsonDecodeResponse['details']['is_distributed'];
                $instanceRole = $isDistributedInstance ? $jsonDecodeResponse['details']['role'][0] : null;
                return [$isDistributedInstance, $instanceRole];
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return [$isDistributedInstance, $instanceRole];
    }

    public function checkForMsspInstance()
    {
        $item = $this->cache->getItem(self::IS_DISTRIBUTED_INSTANCE);
        if ($item->isHit()) {
            return $item->get();
        } else {
            [$isDistributedInstance, $instanceRole] = $this->getDetailsFromDas();
        }
        $item->set($isDistributedInstance);
        $item->tag(CacheConstants::MSSP_TYPE_ROLE);
        $this->cache->save($item);
        return $isDistributedInstance;
    }

    public function getInstanceRole()
    {
        $item = $this->cache->getItem(self::IS_INSTANCE_ROLE);
        if ($item->isHit()) {
            return $item->get();
        } else {
            [$isDistributedInstance, $instanceRole] = $this->getDetailsFromDas();
        }
        $item->set($isDistributedInstance);
        $item->tag(CacheConstants::MSSP_TYPE_ROLE);
        $this->cache->save($item);
        return $instanceRole;
    }

    public function getSelfTenantObject()
    {
        $resourceClass = Tenant::class;
        $tenantRepo = $this->entityManager->getRepository($resourceClass);
        $selfObj = $tenantRepo->findOneBy(["role" => "self"]);
        return $selfObj;
    }

    public function setTenantObject($object)
    {
        $entityClass = get_class($object);
        // Check if Peer Relicable is enabled for class
        if ($this->isPeerReplicable($entityClass)) {
            // Set Self Tenant ID for Objects if not already came as part of requests
            if (!$object->getTenant() || is_null($object->getTenant())) {
                $selfTenantObject = $this->getSelfTenantObject();
                if (is_null($selfTenantObject)) {
                    throw new HttpException(Response::HTTP_BAD_REQUEST, "Self TenantId not defined for instance");
                }
                $object->setTenant($selfTenantObject);
            }
            //Set Owners to Records
            if ($this->isOwnable($entityClass)) {
                $tenantObject = $object->getTenant();
                foreach ($this->tokenStorage->getToken()->getUser()->getTeams() as $owner) {
                    $object->addOwner($owner);
                }
                foreach ($tenantObject->getOwners() as $owner) {
                    $object->addOwner($owner);
                }
            }
            $object->setConflict(false);
        }

        return $object;
    }
}
