<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\Actors\Actor;
use App\Entity\Actors\Avatar;
use App\Entity\Authorization\Team;
use App\Entity\Authorization\Role;
use App\Entity\Core\PgFile;
use App\Entity\Settings\SystemSettings;
use App\Service\SettingsHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;
use Exception;
use Ramsey\Uuid\Uuid;

/**
 * Class ActorConfig
 * @package App\Service\ConfigExportImport
 */
class ActorConfig extends PortingConfig
{
    /** @var SettingsHelper */
    private $settingsHelper;

    const CS_USERTYPE_USER = 2;
    const CS_USERTYPE_SAML = 6;
    const CS_USERTYPE_LDAP = 7;

    public function __construct(SettingsHelper $settingsHelper, EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container)
    {
        $this->settingsHelper = $settingsHelper;
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container);
    }
    /**
     * 
     * @param array $data List of actors
     * @return array
     */
    public function getImportOptions(array $allData, string $key, $tokenType) {
        $data = $allData[$key];
        $repository = $this->loadRepository(Actor::class);
        $options = [];
        foreach ($data as $actor) {
            $existing = $repository->findOneBy(['uuid' => $actor['uuid']]);
            $individualOptions = [];
            $individualOptions['uuid'] = $actor['uuid'];
            $individualOptions['userType'] = $actor['@type'];
            $individualOptions['name'] = $actor['@type'] === 'Person' ? $actor['firstname'] . ' ' . $actor['lastname'] : $actor['name'];
            $individualOptions['existingActorUuid'] = null;
            if ($actor['user']) {
                $existingActorForUuid = $repository->findOneBy(['userId' => $actor['userId']]);
                if ($existingActorForUuid) {
                    $individualOptions['existingActorUuid'] = $existingActorForUuid->getUuid();
                }
                $individualOptions['username'] = $actor['user']['loginid'];
                try {
                    $authUserUri = sprintf('%s/api/auth/users?loginid=%s', $this->container->getParameter('crudhub.uri'), $actor['user']['loginid']);
                    $response = $this->sendRequest($authUserUri, 'get', null, $tokenType);
                    $jsonResponse = json_decode($response, true);
                    $individualOptions['existingUsername'] = array_key_exists('usersresp', $jsonResponse) && count($jsonResponse['usersresp']) > 0 ? $jsonResponse['usersresp'][0]['uuid'] : false;
                    if (!$existingActorForUuid && $individualOptions['existingUsername']) {
                        // If the username matches but the actor uuid does not, use the actor that matches the username
                        $existingActorForUsername = $repository->findOneBy(['userId' => $individualOptions['existingUsername']]);
                        if ($existingActorForUsername) {
                            $individualOptions['existingActorUuid'] = $existingActorForUsername->getUuid();
                        }
                    }
                } catch (Exception $e) {
                    $individualOptions['existingUsername'] = false;
                }
            }
            $individualOptions['include'] = true;
            $individualOptions['existingUserId'] = $existing ? $existing->getUserId() : false;
            $individualOptions['exists'] = $existing || $individualOptions['existingUsername'] || $individualOptions['existingActorUuid'] ? true : false;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $toAdd = [];
        $peopleToUpdate = [];
        $applianceToUpdate = [];
        $roleRepository = $this->loadRepository(Role::class);
        $teamRepository = $this->loadRepository(Team::class);
        $avatarRepository = $this->loadRepository(Avatar::class);
        foreach ($options as $actorOption) {
            if (!$actorOption['include']) {
                // Do not merge skipped items
                continue;
            }
            foreach ($data as $actor) {
                if (!$actor) {
                    continue;
                }
                if ($actorOption['uuid'] == $actor['uuid']) {
                    $uuid = $actor['uuid'];
                    if ($actorOption['existingActorUuid']) {
                        // We cannot change the existing actor uuid for the given userId
                        $uuid = $actorOption['existingActorUuid'];
                    }
                    $moduleName = self::getModuleNameFromType($actor['@type']);
                    $actorIri = sprintf('/api/3/%s/%s', $moduleName, $uuid);
                    $existingRoles = [];
                    foreach ($actor['roles'] as $roleIri) {
                        $roleIriParts = explode('/', $roleIri);
                        $roleUuid = end($roleIriParts);
                        $existingRole = $roleRepository->findOneBy(['uuid' => $roleUuid]);
                        // Only include roles that exist
                        if ($existingRole) {
                            $existingRoles[] = $roleIri;
                        }
                    }
                    $actor['roles'] = $existingRoles;
                    $existingTeams = [];
                    foreach ($actor['teams'] as $teamIri) {
                        $teamIriParts = explode('/', $teamIri);
                        $teamUuid = end($teamIriParts);
                        $existingTeam = $teamRepository->findOneBy(['uuid' => $teamUuid]);
                        // Only include roles that exist
                        if ($existingTeam) {
                            $existingTeams[] = $teamIri;
                        }
                    }
                    $actor['teams'] = $existingTeams;
                    $actor['record_type'] = $moduleName;

                    if ($actor['avatar'] && $actor['avatar_file']) {
                        $pgFile = new PgFile();
                        $pgFile->setFile($actor['avatar_file']);
                        $pgFileManager = $this->container->get('doctrine')->getManagerForClass(PgFile::class);
                        $pgFileManager->persist($pgFile);
                        $pgFileManager->flush();
                        $actor['avatar']['file'] = '/api/3/pgfile/' . $pgFile->getUuid();
                        // Have to change the uuid, otherwise it would fail
                        $actor['avatar']['uuid'] = $uuid;
                        $existingAvatar = $avatarRepository->findOneBy(['uuid' => $uuid]);
                        if ($existingAvatar) {
                            $fullURI = sprintf('%s/api/3/avatars/%s', $this->container->getParameter('crudhub.uri'), $uuid);
                            $result = $this->sendRequest($fullURI, 'put', $actor['avatar'], $tokenType);
                        } else {
                            $fullURI = sprintf('%s/api/3/avatars', $this->container->getParameter('crudhub.uri'));
                            $result = $this->sendRequest($fullURI, 'post', $actor['avatar'], $tokenType);
                        }
                    }
                    if (is_array($actor['avatar'])) {
                        $actor['avatar'] = '/api/3/avatars/' . $actor['avatar']['uuid'];
                    }
                    if (!$actorOption['exists']) {
                        if ($actor['user']['user_type'] == self::CS_USERTYPE_SAML) {
                            $jsonData = $this->sendRequest(sprintf('%s/api/saml/config', $this->container->getParameter('crudhub.uri')), 'get', null, $tokenType);
                            $result = json_decode($jsonData, true);
                            if (!$result['samlEnabled']) {
                                $actor['user']['user_type'] = self::CS_USERTYPE_USER;
                                $actor['userType'] = null;
                            }
                        } else if ($actor['user']['user_type'] == self::CS_USERTYPE_LDAP) {
                            $authConfigUri = sprintf('%s/api/auth/config?section=LDAP', $this->container->getParameter('crudhub.uri'));
                            $jsonData = $this->sendRequest($authConfigUri, 'get', null, $tokenType);
                            $result = json_decode($jsonData, true);
                            foreach ($result['hydra:member'] as $ldapSetting) {
                                if ($ldapSetting['key'] == 'ldap_enabled' && !$ldapSetting['value']) {
                                    $actor['user']['user_type'] = self::CS_USERTYPE_USER;
                                    $actor['user']['bind_name'] = null;
                                    $actor['userType'] = null;
                                }
                            }
                        }
                        $toAdd[] = $actor;
                        break;
                    }
                    unset($actor['uuid']);
                    unset($actor['csActive']);
                    $actor['@id'] = $actorIri;
                    // If the user exists, we do not overwrite the user details
                    $actor['userId'] = $actorOption['existingUserId'] ? $actorOption['existingUserId'] : $actorOption['existingUsername'];
                    if ($moduleName === 'people') {
                        $peopleToUpdate[] = $actor;
                    } else {
                        $applianceToUpdate[] = $actor;
                    }
                    break;
                }
            }
        }
        $results = [];
        $authUserUri = sprintf('%s/api/auth/users', $this->container->getParameter('crudhub.uri'));
        foreach ($toAdd as $item) {
            // First add the user
            if ($item['user']) {
                $body = [
                    'credentials' => [
                        'loginid' => $item['user']['loginid']
                    ],
                    'status' => 2,
                    'type' => $item['user']['user_type'],
                    'domain' => $item['user']['domain'],
                    'nfa' => array_key_exists('nfa', $item['user']) ? $item['user']['nfa'] : [],
                    'bind_name' => $item['user']['bind_name'],
                    'tenant' => $item['user']['tenant']
                ];
                $response = $this->sendRequest($authUserUri, 'post', $body, $tokenType);
                $jsonResponse = json_decode($response, true);
                $item['userId'] = $jsonResponse['uuid'];
                $item['csActive'] = false;
                unset($item['user']);
            }
            $fullURI = sprintf('%s/api/3/%s', $this->container->getParameter('crudhub.uri'), $actor['record_type']);
            $result = $this->sendRequest($fullURI, 'post', $item, $tokenType);
            $results[] = $result;
            // Finally, add user settings
            if ($item['@userSettings']) {
                $iri = sprintf('/api/3/%s/%s', $actor['record_type'], $item['uuid']);
                $userSettings = $this->getUserSettingsFromActorIRI($iri);
                $this->settingsHelper->saveUserSettings($userSettings, $item['@userSettings']);
            }
        }
        if (count($peopleToUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/people', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $peopleToUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
            foreach ($peopleToUpdate as $item) {
                if ($item['@userSettings']) {
                    $userSettings = $this->getUserSettingsFromActorIRI($item['@id']);
                    $this->settingsHelper->saveUserSettings($userSettings, $item['@userSettings']);
                }
            }
            
        }

        if (count($applianceToUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/appliances', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $applianceToUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        $actors = $this->getResourceFromIRI($options, Actor::class, true, true);
        $authUserUri = sprintf('%s/api/auth/users', $this->container->getParameter('crudhub.uri'));
        $response = $this->sendRequest($authUserUri, 'get', null, $tokenType);
        $userList = json_decode($response, true)['usersresp'];
        foreach ($actors as $index => $actor) {
            $user = self::findBy($userList, 'uuid', $actor['userId'], true);
            foreach ($actor as $key => $value) {
                if (is_array($value) && !in_array($key, ['user', 'roles', 'teams'])) {
                    unset($actors[$index][$key]);
                }
            }
            $moduleName = self::getModuleNameFromType($actor['@type']);
            $actorIri = sprintf('/api/3/%s/%s', $moduleName, $actor['uuid']);
            $userSettings = $this->getUserSettingsFromActorIRI($actorIri);
            if ($userSettings) {
                $actors[$index]['@userSettings'] = $userSettings->getPublicValues();
            }
            if ($user) {
                $actors[$index]['user'] = $user;
            }

            if ($actor['avatar']) {
                $avatar = end($this->getResourceFromIRI([$actor['avatar']], Avatar::class));
                $fileRepository = $this->entityManager->getRepository(PgFile::class);
                $fileIriParts = explode('/', $avatar['file']);
                $pgFile = $fileRepository->findOneBy(['uuid' => end($fileIriParts)]);
                $actors[$index]['avatar_file'] = stream_get_contents($pgFile->getFile());
                $actors[$index]['avatar'] = $avatar;
            }
        }
        return $actors;
    }

    private function getUserSettingsFromActorIRI($actorIRI) {
        return $this->settingsHelper->getUserSettingsFromIRI($actorIRI);
    }

    private static function getModuleNameFromType($type) {
        return $type == 'Person' ? 'people' : 'appliances';
    }
}
