<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\Custom\Agent;
use App\Entity\Workflow\WorkflowCollection;
use App\Providers\PermissionsDictionaryBuilder;
use App\Service\ConfigExportImport\PlaybookConfig;
use App\Security\Voter\FsrVoter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

/**
 * Class ConnectorConfig
 * @package App\Service\ConfigExportImport
 */
class ConnectorConfig extends PortingConfig
{
    /** @var fsrVoter */
    private $fsrVoter;

    /** @var PlaybookConfig */
    private $playbookConfig;

    public function __construct(FsrVoter $fsrVoter, ImportService $importService, EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container, PlaybookConfig $playbookConfig)
    {
        $this->fsrVoter = $fsrVoter;
        $this->importService = $importService;
        $this->playbookConfig = $playbookConfig;
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container);
    }

    /**
     *
     * @param array $data Playbook object containing collections & global vars
     * @return array
     */
    public function getImportOptions(array $allData, string $key, $tokenType)
    {
        $data = $allData[$key];
        // at least one of these permissions
        if (!($this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . PermissionsDictionaryBuilder::MODULE_CONNECTORS)) &&
        !($this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.' . PermissionsDictionaryBuilder::MODULE_CONNECTORS)) && 
        !($this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_CREATE . '.' . PermissionsDictionaryBuilder::MODULE_CONNECTORS)) ){
            return [];
        }
        $options = [];
        $existingConnectors = $this->getConnectors($tokenType);
        $existingConnectorNames = array_column($existingConnectors, 'name');
        $options = [];
        foreach ($data as $connector) {
            $existing = in_array($connector['name'], $existingConnectorNames);
            $individualOptions = [];
            $individualOptions['name'] = $connector['name'];
            $individualOptions['label'] = $connector['label'];
            $individualOptions['version'] = $connector['version'];
            $individualOptions['rpm_name'] = $connector['rpm_name'];
            $individualOptions['configurationCount'] = count($connector['configurations']);
            if (count($connector['configurations']) > 0) {
                $individualOptions['includeConfigurations'] = true;
            }
            $individualOptions['exists'] = $existing;
            if ($existing) {
                foreach ($existingConnectors as $existingConnector) {
                    if ($existingConnector['name'] == $connector['name']) {
                        $individualOptions['existingId'] = $existingConnector['id'];
                        $existingVersion = str_replace('.', '', $existingConnector['version']);
                        $version = str_replace('.', '', $connector['version']);
                        if ($existingVersion >= $version) {
                            $individualOptions['includeRpm'] = false;
                            $individualOptions['locked'] = true;
                        } else {
                            $individualOptions['includeRpm'] = !!$connector['rpm_name'];
                            $individualOptions['existingVersion'] = $existingConnector['version'];
                        }
                        break;
                    }
                }
            } else if ($connector['rpm_name']) {
                $individualOptions['includeRpm'] = true;
            }

            $options[] = $individualOptions;
        }
        return $options;
        
        return $options;
    }

    private function getConnectors($tokenType='JWT')
    {
        $token = $this->getTokenFromStorage();
        $endpoint = '/connector_details/?ordering=label&page_size=10000&$export=true&exclude=operation';
        if ($token) {
            $uri = sprintf('%s/api/integration%s', $this->container->getParameter('crudhub.uri'), $endpoint);
            $response = $this->sendRequest($uri, 'post', null, $tokenType);
            $jsonDecodeResponse = json_decode($response, true);
            return array_key_exists('data', $jsonDecodeResponse) ? $jsonDecodeResponse['data'] : $jsonDecodeResponse;
        } else {
            $uri = $this->container->getParameter('integrations.uri') . $endpoint;
            return $this->sendIntegrationRequest($uri, 'post');
        }
    }

    private function getDataImports($tokenType='JWT', $configurationUuid)
    {
        $token = $this->getTokenFromStorage();
        $endpoint = '/data-import/?configuration=' . $configurationUuid;
        if ($token) {
            $uri = sprintf('%s/api/integration%s', $this->container->getParameter('crudhub.uri'), $endpoint);
            $response = $this->sendRequest($uri, 'get', null, $tokenType);
            $jsonDecodeResponse = json_decode($response, true);
            return array_key_exists('data', $jsonDecodeResponse) ? $jsonDecodeResponse['data'] : $jsonDecodeResponse;
        } else {
            $uri = $this->container->getParameter('integrations.uri') . $endpoint;
            return $this->sendIntegrationRequest($uri, 'get');
        }
    }
    
    
    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType)
    {
        $token = $this->getTokenFromStorage();
        foreach ($options as $option) {
            foreach ($data as $connector) {
                if ($connector['rpm_name'] == $option['rpm_name']) {
                    $connectorId = array_key_exists('existingId', $option) ? $option['existingId'] : null;
                    // Install & wait if not exists
                    if ($option['includeRpm']) {
                        if ($token) {
                            $this->sendRequest(sprintf('%s/api/integration/install-connector/?format=json', $this->container->getParameter('crudhub.uri')), 'post', $connector, $tokenType);
                        } else {
                            $this->sendIntegrationRequest(sprintf('%s/install-connector/?format=json', $this->container->getParameter('integrations.uri')), 'post', $connector);
                        }
                        $status = 'In-Progress';
                        while ($status == 'In-Progress') {
                            sleep(5);
                            $iri = sprintf('connectors/%s/%s/?format=json', $connector['name'], $connector['version']);
                            if ($token) {
                                $jsonData = $this->sendRequest(sprintf('%s/api/integration/%s', $this->container->getParameter('crudhub.uri'), $iri), 'post', $connector, $tokenType);
                            } else {
                                $jsonData = $this->sendIntegrationRequest(sprintf('%s/%s', $this->container->getParameter('integrations.uri'), $iri), 'post', $connector);
                            }
                            $result = json_decode($jsonData, true);
                            $status = $result['status'];
                            $connectorId = $result['id'];
                        }
                    } else {
                        $iri = sprintf('connectors/%s/%s/?format=json', $connector['name'], $connector['version']);
                        if ($token) {
                            $jsonData = $this->sendRequest(sprintf('%s/api/integration/%s', $this->container->getParameter('crudhub.uri'), $iri), 'post', $connector, $tokenType);
                        } else {
                            $jsonData = $this->sendIntegrationRequest(sprintf('%s/%s', $this->container->getParameter('integrations.uri'), $iri), 'post', $connector);
                        }
                        $result = json_decode($jsonData, true);
                    }
                    if (!$option['includeConfigurations']) {
                        continue;
                    }
                    //Added sleep to handle the connector dependency install time.
                    sleep(10);
                    // First, delete existing configs
                    foreach ($connector['configurations'] as $configuration) {
                        foreach ($result['configuration'] as $existingConfiguration) {
                            if ($existingConfiguration['config_id'] == $configuration['config_id'] || $existingConfiguration['name'] == $configuration['name']) {
                                // Delete existing config
                                if ($token) {
                                    $this->sendRequest(sprintf('%s/api/integration/configuration/%s/?format=json', $this->container->getParameter('crudhub.uri'), $existingConfiguration['id']), 'delete', null, $tokenType);
                                } else {
                                    $this->sendIntegrationRequest(sprintf('%s/configuration/%s/?format=json', $this->container->getParameter('integrations.uri'), $existingConfiguration['id']), 'delete', null);
                                }
                                break;
                            }
                        }
                    }

                    // Next, ingestion playbooks need to go in
                    if ($connector['ingestionPlaybooks']) {
                        $allData = [
                            'playbooks' => $connector['ingestionPlaybooks']
                        ];
                        $playbookOptions = $this->playbookConfig->getImportOptions($allData, 'playbooks', $tokenType);
                        $this->playbookConfig->import($allData['playbooks'], $playbookOptions, $tokenType);
                    }

                    // Install configurations afterwards
                    foreach ($connector['configurations'] as $configuration) {
                        $configuration['connector'] = $connectorId;
                        if (array_key_exists('agent', $configuration)) {
                            unset($configuration['agent']);
                        }
                        try {
                            if ($token) {
                                $this->sendRequest(sprintf('%s/api/integration/configuration/?format=json', $this->container->getParameter('crudhub.uri')), 'post', $configuration, $tokenType);
                            } else {
                                $this->sendIntegrationRequest(sprintf('%s/configuration/?format=json', $this->container->getParameter('integrations.uri')), 'post', $configuration);
                            }
                        } catch (\Exception $exception) {
                            // Config already exists, ignore
                            continue;
                        }
                    }

                    if (!array_key_exists('dataImports', $connector)) {
                        continue;
                    }

                    foreach ($connector['dataImports'] as $dataImport) {
                        try {
                            if ($token) {
                                $scheduleId = $this->saveSchedule($dataImport['schedule'], $tokenType);
                                $dataImport['metadata']['scheduleId'] = $scheduleId;
                                $this->sendRequest(sprintf('%s/api/integration/data-import/?format=json', $this->container->getParameter('crudhub.uri')), 'post', $dataImport, $tokenType);
                            } else {
                                $this->sendIntegrationRequest(sprintf('%s/data-import/?format=json', $this->container->getParameter('integrations.uri')), 'post', $dataImport);
                            }
                        } catch (\Exception $exception) {
                            // Data import already exists, ignore
                            continue;
                        }
                    }
                }
            }
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT')
    {
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.' . PermissionsDictionaryBuilder::MODULE_CONNECTORS)) {
            return [];
        }
        $data = [];
        if (count($options) > 0) {
            $connectors = $this->getConnectors($tokenType);
        }
        foreach ($options as $option) {
            foreach ($connectors as $connector) {
                // Load ingestion playbooks
                $rpmName = sprintf('cyops-connector-%s-%s', $connector['name'], $connector['version']);
                if ($option['value'] == $rpmName) {
                    $connectorObject = [
                        'name' => $connector['name'],
                        'label' => $connector['label'],
                        'category' => $connector['category'],
                        'description' => $connector['description'],
                        'publisher' => $connector['publisher'],
                        'version' => $connector['version'],
                        'rpm_name' => null,
                        'configurations' => [],
                        'dataImports' => []
                    ];
                    if ($option['rpm']) {
                        $connectorObject['rpm_name'] = $rpmName;
                    }
                    if ($option['configurations']) {
                        $connectorObject['configurations'] = $connector['configuration'];
                        if (count($connector['configuration']) > 0) {
                            $configIds = array_column($connector['configuration'], 'config_id');
                            $options = [
                                'collections' => $configIds,
                                'globalVariables' => []
                            ];
                            $ingestionPlaybooks = $this->playbookConfig->export($options, $tokenType);
                            if ($ingestionPlaybooks) {
                                $connectorObject['ingestionPlaybooks'] = $ingestionPlaybooks;
                            }
                            foreach ($configIds as $configId) {
                                $configDataImports = $this->getDataImports($tokenType, $configId);
                                foreach ($configDataImports as $dataImport) {
                                    if ($dataImport['metadata']['scheduleId']) {
                                        $dataImport['schedule'] = $this->loadSchedule($dataImport['metadata']['scheduleId'], $tokenType);
                                    }
                                    $connectorObject['dataImports'][] = $dataImport;
                                }
                            }
                        }
                    }
                    
                    $data[] = $connectorObject;
                    break;
                }
            }
        }
        return $data;
    }
}
