<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use App\Entity\Authorization\Role;
use App\Entity\ViewTemplate\Dashboard;
use App\Providers\PermissionsDictionaryBuilder;
use App\Security\Voter\FsrVoter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

/**
 * Class DashboardConfig
 * @package App\Service\ConfigExportImport
 */
class DashboardConfig extends PortingConfig
{
    /** @var fsrVoter */
    private $fsrVoter;

    public function __construct(FsrVoter $fsrVoter, EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container)
    {
        $this->fsrVoter = $fsrVoter;
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container);
    }
    
    /**
     * @param array $allData
     * @param string $key
     * @return void
     */
    public function getImportOptions(array $allData, string $key, $tokenType)
    {
        $data = $allData[$key];
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_CREATE . '.dashboard') || 
            !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.dashboard') || 
            !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ. '.dashboard')) {
            return [];
        }
        $repository = $this->loadRepository(Dashboard::class);
        $roleRepository = $this->loadRepository(Role::class);
        $options = [];
        foreach ($data as $dashboard) {
            $existingUuids = $repository->findOneBy(['uuid' => $dashboard['uuid']]);
            $existingDisplayNames = $repository->findOneBy(['displayName' => $dashboard['displayName']]);
            if (array_key_exists('roleIds', $dashboard)) {
                $roleUuids = $dashboard['roleIds'];
            } else {
                $roleUuids = [];
                foreach ($dashboard['roles'] as $roleIri) {
                    $parts = explode('/', $roleIri);
                    $roleUuid = end($parts);
                    $roleUuids[] = $roleUuid;
                }
            }

            $roles = $roleRepository->findBy(['uuid' => $roleUuids]);
            $foundRoles = [];
            foreach ($roles as $role) {
                $foundRoles[] = [
                    '@id' => '/api/3/roles/' . $role->getUuid(),
                    'name' => $role->getName()
                ];
            }
            
            $individualOptions = [];
            $individualOptions['uuid'] = $existingDisplayNames ? $existingDisplayNames->getUuid() : $dashboard['uuid'];
            $individualOptions['name'] = $dashboard['displayName'];
            $individualOptions['roles'] = $foundRoles;
            $individualOptions['include'] = true;
            $individualOptions['exists'] = $existingUuids || $existingDisplayNames ? true : false;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $dashboardsToAdd = [];
        $dashboardsToUpdate = [];
        foreach ($options as $dashboardOption) {
            if (!$dashboardOption['include']) {
                // Do not merge skipped items
                continue;
            }
            foreach ($data as $dashboard) {
                if ($dashboardOption['uuid'] == $dashboard['uuid']) {
                    unset($dashboard['userId']);
                    $dashboard['roles'] = array_column($dashboardOption['roles'], '@id');
                    $dashboard['filterArray'] = isset($dashboard['config']['filterArray']) ? $dashboard['config']['filterArray'] : $dashboard['filterArray'];
                    unset($dashboard['roleIds']);
                    if (!$dashboardOption['exists']) {
                        $dashboardsToAdd[] = $dashboard;
                        break;
                    }
                    $dashboard['@id'] = sprintf('/api/3/dashboard/%s', $dashboard['uuid']);
                    unset($dashboard['uuid']);
                    $dashboardsToUpdate[] = $dashboard;
                    break;
                }
            }
        }
        $results = [];
        foreach ($dashboardsToAdd as $dashboard) {
            $fullURI = sprintf('%s/api/3/dashboard', $this->container->getParameter('crudhub.uri'));
            $result = $this->sendRequest($fullURI, 'post', $dashboard, $tokenType);
            $results[] = $result;
        }
        if (count($dashboardsToUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/dashboard', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $dashboardsToUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.dashboard')) {
            return [];
        }
        return $this->getResourceFromIRI($options, Dashboard::class, true);
    }
}
