<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */

namespace App\Service\ConfigExportImport;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

/**
 * Interface PortingConfig
 * @package App\Service
 */
abstract class PortingConfig
{
    const METHOD_MAP = [
        "post" => "create",
        "put" => "update",
        "delete" => "delete"
    ];

    /** @var EntityManagerInterface */
    protected $entityManager;

    /** @var NormalizerInterface */
    protected $normalizer;

    /** @var ContainerInterface */
    protected $container;

    /** @var TokenStorageInterface */
    private $tokenStorage;

    public function __construct(EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container){
        $this->entityManager = $entityManager;
        $this->normalizer = $normalizer;
        $this->tokenStorage = $tokenStorage;
        $this->container = $container;
    }

    /**
     * Generates import options from import data
     *
     * @param array $data
     * @return array
     */
    abstract public function getImportOptions(array $allData, string $key, $tokenType);

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    abstract public function import(array $data, array $options, $tokenType);

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    abstract public function export(array $options, $tokenType='JWT');

    protected function getTokenFromStorage(){
        return $this->tokenStorage->getToken();
    }

    protected function loadRepository($resourceClass){
        return $this->entityManager->getRepository($resourceClass);
    }

    protected function getResourceFromIRI($iris, $className, $relationships = false, $export = true, $versions = false, $ignoreDepth = false) {
        $repository = $this->entityManager->getRepository($className);
        if ($versions) {
            $_GET['$versions'] = true;
            $_GET['$includeData'] = true;
        }
        $context = $this->getDefaultNormalizationContext($className, $export, $relationships, $ignoreDepth);
        $exportRecords = [];
        foreach ($iris as $iri) {
            $parts = explode('/', $iri);
            //TODO check if we need team rbac here. accordingly use getItemFromIri instead
            $record = $repository->findOneBy(['uuid' => end($parts)]);
            $normalizedObject = $this->normalizer->normalize($record, 'jsonld', $context);
            if ($normalizedObject) {
                $exportRecords[] = $normalizedObject;
            }
        }
        return $exportRecords;
    }

    protected function sendRequest($uri, $method, $data = null, $tokenType='JWT') {
        $token = $this->tokenStorage->getToken();
        if ($token && $tokenType == 'JWT') {
            $client = new Client();
        } else {
            $client = $this->container->get("app.proxy.client.crudhub");
        }
        $headers = array(
            "Content-type" => "application/json",
            "Authorization" => $token ? sprintf("Bearer %s", $token->getCredentials()) : null
        );
        $request = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
        try {
            $response = $client->send($request, ['verify' => false]);
            $statusCode = $response->getStatusCode();
            if ($statusCode < 400) {
                //$jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return $response->getBody()->getContents();
            } else {
                throw new \Exception($uri.'\n'.$response->getBody());
            }
        } catch (\Exception $exception) {
            $errCode = $exception->getCode();
            if ($errCode == 403) {
                $iri = str_replace($this->container->getParameter('crudhub.uri'), '', $uri);
                $module = explode('/', $iri)[4];
                throw new \Exception(sprintf('You do not have %s access on %s module', self::METHOD_MAP[$method], $module));
            }
            elseif ($errCode == 404) {
                $iri = str_replace($this->container->getParameter('crudhub.uri'), '', $uri);
                $message = $data ? json_encode($data) : "{}";
                throw new \Exception(sprintf('The Requested Resource %s (%s) Does not exists ', $iri, $method));
            }
            elseif ($errCode == 409) {
                $message = json_decode($exception->getResponse()->getBody(), true);
                throw new \Exception(sprintf($message['hydra:description']));
            }
            $message = $exception->getResponse() ? $exception->getResponse()->getBody() : $exception->getMessage();
            throw new \Exception(sprintf('%d: %s', $errCode, $message));
        }
    }
    
    protected function sendIntegrationRequest($uri, $method, $data = null)
    {
        $client = $this->container->get("app.proxy.client.integrations");
        $headers = array(
            "Content-type" => "application/json",
            "Authorization" => null
        );
        $request = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
        try {
            $response = $client->send($request, ['verify' => false]);
            $statusCode = $response->getStatusCode();
            if ($statusCode < 400) {
                //$jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return $response->getBody();
            } else {
                throw new \Exception($response->getBody());
            }
        } catch (\Exception $exception) {
            $errCode = $exception->getCode();
            $message = $exception->getResponse() ? $exception->getResponse()->getBody() : $exception->getMessage();
            throw new \Exception(sprintf('%d: %s', $errCode, $message));
        }
    }

    /**
     * Finds the value in the array by key
     *
     * @param array $array
     * @param string $key
     * @param string $value
     * @return mixed
     */
    protected static function findBy(array $array, string $key, string $value, $caseInsensitive = false)
    {
        foreach ($array as $item) {
            if (!$caseInsensitive && $item[$key] == $value) {
                return $item;
            } else if ($caseInsensitive && strtolower($item[$key]) == strtolower($value)) {
                return $item;
            }
        }
        return null;

    }

    protected function sealabRequest($uri, $method, $data = null)
    {
        $headers = array(
            "Content-type" => "application/json",
        );
        $workflowRequest = new GuzzleRequest($method, $uri, $headers, $data ? json_encode($data) : null);
        $client = $this->container->get("app.proxy.client.workflow");
        try {
            $response = $client->send($workflowRequest, ['verify' => false]);
            $statusCode = $response->getStatusCode();
            if ($statusCode < 400) {
                $jsonDecodeResponse = json_decode($response->getBody()->getContents(), true);
                return array_key_exists('hydra:member', $jsonDecodeResponse) ? $jsonDecodeResponse['hydra:member'] : $jsonDecodeResponse;
            } else {
                throw new \Exception($uri.'\n'.$response->getBody());
            }
        } catch (\Exception $exception) {
            $errVar = json_decode($exception->getResponse()->getBody()->read(2048), true);
            $errMsg['message'] = $errVar['Error'] ? isset($errVar['Error']) : $errVar['error'];
            $errCode = $exception->getCode();
            throw (sprintf('%d: %s', $errCode, $errMsg));
        }
    }

    protected function getDefaultNormalizationContext(string $resourceClass, bool $export = true, bool $relationships = false, bool $ignoreDepth = false){
        return [
        'export' => $export,
        'relationships' => $relationships,
        'ignore_field_authorization' => true,
        'ignore_authorization' => true,
        'ignore_depth' => $ignoreDepth,
        'resource_class' => $resourceClass,
        'item_operation_name' => 'get',
        'collection_operation_name' => 'get'];
    }

    protected function saveSchedule($schedule, $tokenType) {
        $endpoint = '/api/scheduled/?format=json';
        $method = 'post';
        // Check if schedule exists
        $getEndpoint = sprintf('/api/scheduled/?format=json&task=workflow.tasks.periodic_task&name=%s', $schedule['name']);
        try {
            $getResult = $this->sendPlaybookRequest($getEndpoint, 'get', null, $tokenType);
            if (count($getResult) > 0) {
                $method = 'put';
                $endpoint = sprintf('/api/scheduled/%s/?format=json', $getResult[0]['id']);
            }
        } catch (\Exception $e) {
            // Does not exist
        }
        $result = $this->sendPlaybookRequest($endpoint, $method, $schedule, $tokenType);
        return $result['id'];
    }

    protected function loadSchedule($scheduleId, $tokenType) {
        $endpoint = sprintf('/api/scheduled/%s/?format=json', $scheduleId);
        try {
            $getResult = $this->sendPlaybookRequest($endpoint, 'get', null, $tokenType);
            return $getResult;
        } catch (\Exception $e) {
            // Does not exist
        }
        return null;
    }

    private function sendPlaybookRequest($endpoint, $method, $data, $tokenType) {
        $tokenStorage = $this->container->get('security.token_storage');
        $token = $tokenStorage->getToken();
        if ($token) {
            $uri = $this->container->getParameter('crudhub.uri') . '/api/wf' . $endpoint;
            $response = $this->sendRequest($uri, $method, $data, $tokenType);
            $jsonDecodeResponse = json_decode($response, true);
            return array_key_exists('hydra:member', $jsonDecodeResponse) ? $jsonDecodeResponse['hydra:member'] : $jsonDecodeResponse;
        } else {
            $uri = $this->container->getParameter('workflow.uri') . $endpoint;
            return $this->sealabRequest($uri, 'get');
        }
    }

}
