<?php
/* Copyright start
  Copyright (C) 2008 - 2021 Fortinet Inc.
  All rights reserved.
  FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
  Copyright end */


namespace App\Service\ConfigExportImport;

use App\Entity\Authorization\Role;
use App\Entity\ViewTemplate\Reporting;
use App\Providers\PermissionsDictionaryBuilder;
use App\Security\Voter\FsrVoter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;
use Symfony\Component\Serializer\Normalizer\NormalizerInterface;

/**
 * Class ReportConfig
 * @package App\Service\ConfigExportImport
 */
class ReportConfig extends PortingConfig
{
    /** @var fsrVoter */
    private $fsrVoter;

    public function __construct(FsrVoter $fsrVoter, EntityManagerInterface $entityManager, NormalizerInterface $normalizer, 
        TokenStorageInterface $tokenStorage, ContainerInterface $container)
    {
        $this->fsrVoter = $fsrVoter;
        parent::__construct($entityManager, $normalizer, $tokenStorage, $container);
    }

    /**
     * @param array $allData
     * @param string $key
     * @return void
     */
    public function getImportOptions(array $allData, string $key, $tokenType)
    {
        $data = $allData[$key];
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_CREATE . '.reporting') ||
         !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_UPDATE . '.reporting') || 
         !$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.reporting')) {
            return [];
        }
        $repository = $this->loadRepository(Reporting::class);
        $roleRepository = $this->loadRepository(Role::class);
        $options = [];
        foreach ($data as $report) {
            $existing = $repository->findOneBy(['uuid' => $report['uuid']]);
            if ($report['roleIds']) {
                $roleUuids = $report['roleIds'];
            } else {
                $roleUuids = [];
                foreach ($report['roles'] as $roleIri) {
                    $parts = explode('/', $roleIri);
                    $roleUuid = end($parts);
                    $roleUuids[] = $roleUuid;
                }
            }

            $roles = $roleRepository->findBy(['uuid' => $roleUuids]);
            $foundRoles = [];
            foreach ($roles as $role) {
                $foundRoles[] = [
                    '@id' => '/api/3/roles/' . $role->getUuid(),
                    'name' => $role->getName()
                ];
            }
            $individualOptions = [];
            $individualOptions['uuid'] = $report['uuid'];
            $individualOptions['name'] = $report['displayName'];
            $individualOptions['roles'] = $foundRoles;
            $individualOptions['include'] = true;
            $individualOptions['exists'] = $existing ? true : false;

            $options[] = $individualOptions;
        }
        return $options;
    }

    /**
     * Imports the config data using the specified options
     *
     * @param array $data
     * @param array $options
     * @return mixed
     */
    public function import(array $data, array $options, $tokenType) {
        $reportsToAdd = [];
        $reportsToUpdate = [];
        foreach ($options as $reportOption) {
            if (!$reportOption['include']) {
                // Do not merge skipped items
                continue;
            }
            foreach ($data as $report) {
                if ($reportOption['uuid'] == $report['uuid']) {
                    unset($report['userId']);
                    $report['roles'] = array_column($reportOption['roles'], '@id');
                    $report['filterArray'] = isset($report['config']['filterArray']) ? $report['config']['filterArray'] : $report['filterArray'];

                    unset($report['roleIds']);
                    if (!$reportOption['exists']) {
                        $reportsToAdd[] = $report;
                        break;
                    }
                    $report['@id'] = sprintf('/api/3/reporting/%s', $report['uuid']);
                    unset($report['uuid']);
                    $reportsToUpdate[] = $report;
                    break;
                }
            }
        }
        $results = [];
        foreach ($reportsToAdd as $report) {
            $fullURI = sprintf('%s/api/3/reporting', $this->container->getParameter('crudhub.uri'));
            $result = $this->sendRequest($fullURI, 'post', $report, $tokenType);
            $results[] = $result;
        }
        if (count($reportsToUpdate) > 0) {
            $fullURI = sprintf('%s/api/3/update/reporting', $this->container->getParameter('crudhub.uri'));
            $body = [
              "data" => $reportsToUpdate
            ];
            $result = $this->sendRequest($fullURI, 'put', $body, $tokenType);
            $results[] = $result;
        }
        
        if (in_array(false, $results)) {
            return false;
        }
        return true;
    }

    /**
     * Exports the config using the specified options
     *
     * @param array $options
     * @return array
     */
    public function export(array $options, $tokenType='JWT') {
        if (!$this->fsrVoter->isGranted(PermissionsDictionaryBuilder::PERM_CRUD_READ . '.reporting')) {
            return [];
        }
        return $this->getResourceFromIRI($options, Reporting::class, true);
    }
}
