<?php
/* Copyright start
Copyright (C) 2008 - 2021 Fortinet Inc.
All rights reserved.
FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
Copyright end */

namespace App\Service;

use ApiPlatform\Core\Exception\ItemNotFoundException;
use ApiPlatform\Core\Metadata\Resource\Factory\ResourceMetadataFactoryInterface;
use App\Constants\CacheConstants;
use App\Entity\ModelMetadata\ModelMetadata;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Inflector\Inflector as LegacyInflector;
use Doctrine\Inflector\InflectorFactory;
use Psr\Container\ContainerInterface;
use Symfony\Component\Cache\Adapter\FilesystemTagAwareAdapter;
use Symfony\Contracts\Cache\TagAwareCacheInterface;

class MetadataHelper
{
    /* @var TagAwareCacheInterface */
    private $cache;

    /* @var EntityManager */
    private $entityManager;

    private $resourceMetadataFactory;

    /** @var ContainerInterface */
    private $container;

    private $utilityService;

    public function __construct(
        TagAwareCacheInterface $cache,
        EntityManagerInterface $entityManager,
        ResourceMetadataFactoryInterface $resourceMetadataFactory,
        ContainerInterface $container,
        UtilityService $utilityService
    ) {
        $this->cache = $cache;
        $this->entityManager = $entityManager;
        $this->resourceMetadataFactory = $resourceMetadataFactory;
        $this->utilityService = $utilityService;
        $this->container = $container;
        $this->inflector = InflectorFactory::create()->build();
    }

    public function getTypeFromShortName($resourceShortName)
    {
        $entityMap = $this->cache->getItem(CacheConstants::ENTITY_SHORTNAME_TYPENAME_MAP);
        if (!($entityMap->isHit())) {
            $entityMapLocal = array();
            $modelMetaDataRepository = $this->entityManager->getRepository(ModelMetadata::class);
            $modelMetaDatas = $modelMetaDataRepository->findAll();
            foreach ($modelMetaDatas as $modelMetaData) {
                $singular = $modelMetaData->getDescriptions()['singular'];
                $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
                $escapedSingular = LegacyInflector::classify($escapedSingular);
                $type = $modelMetaData->getType();
                $entityMapLocal[$escapedSingular] = $type;
            }
            $systemFixturesFile = file_get_contents('/opt/cyops-api/src/StaticFile/SystemFixtures.json');
            $systemFixtures = json_decode($systemFixturesFile, true);
            foreach ($systemFixtures as $systemFixture) {
                $singular = $systemFixture['descriptions']['singular'];
                if ($singular == "Playbook") {
                    $singular = "Workflow";
                } else if ($singular == "Playbook Collection") {
                    $singular = "Workflow Collection";
                }
                $escapedSingular = preg_replace('/[^a-z0-9]+/i', '', $singular);
                $type = $systemFixture['type'];
                $entityMapLocal[$escapedSingular] = $type;
            }
            $entityMap->set($entityMapLocal);
            $entityMap->tag(CacheConstants::TAG_MODEL_METADATA);
            $this->cache->save($entityMap);
            $entityMap = $this->cache->getItem(CacheConstants::ENTITY_SHORTNAME_TYPENAME_MAP);
        }
        $typeMap = $entityMap->get();
        if (array_key_exists($resourceShortName, $typeMap)) {
            return $typeMap[$resourceShortName];
        } else {
            throw new ItemNotFoundException('no model found with short name ' . $resourceShortName);
        }
    }

    public function getTypeFromClassName(String $resourceClass)
    {
        $resourceMetadata = $this->resourceMetadataFactory->create($resourceClass);
        if (array_key_exists('fsr_type', $resourceMetadata->getAttributes())) {
            $resourceType = $resourceMetadata->getAttributes()['fsr_type'];
        }

        if (is_null($resourceType)) {
            throw new ItemNotFoundException('fsr_type not defined for class ' . $resourceClass);
        }
        return $resourceType;
    }

    public function getModuleFromClassName(String $resourceClass)
    {
        $resourceMetadata = $this->resourceMetadataFactory->create($resourceClass);
        if (array_key_exists('fsr_module', $resourceMetadata->getAttributes())) {
            $moduleType = $resourceMetadata->getAttributes()['fsr_module'];
        }
        if (is_null($moduleType)) {
            throw new ItemNotFoundException('fsr_module not defined for class ' . $resourceClass);
        }
        return $moduleType;
    }

    public function getEntityFields(string $resourceClass)
    {
        $entityFieldMapCache = $this->cache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP);
        if (!$entityFieldMapCache->isHit()) {
            $this->createModuleMapCache();
            $entityFieldMapCache = $this->cache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP);
        }
        $entityFieldMap = $entityFieldMapCache->get()[$resourceClass] ?? null;
        return $entityFieldMap;
    }

    public function createModuleMapCache()
    {
        $cacehDir = sprintf(CacheConstants::RELATIVE_METADATA_CACHE_PATH, $this->container->getParameter('kernel.root_dir'));
        $fcache = new FilesystemTagAwareAdapter('', 0, $cacehDir);
        $item = $fcache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        if (!$item->isHit()) {
            $this->getModuleMapFileCache($fcache);
        }

        $mFieldMap = $this->cache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP);
        $mFieldMap->set($fcache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP)->get());
        $mFieldMap->tag(CacheConstants::TAG_MODEL_METADATA);
        $this->cache->save($mFieldMap);

        $mModuleMap = $this->cache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        $mModuleMap->set($fcache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP)->get());
        $mModuleMap->tag(CacheConstants::TAG_MODEL_METADATA);
        $this->cache->save($mModuleMap);
    }

    public function getModuleMapFileCache(FilesystemTagAwareAdapter $fcache = null)
    {
        if (is_null($fcache)) {
            $cacehDir = sprintf(CacheConstants::RELATIVE_METADATA_CACHE_PATH, $this->container->getParameter('kernel.root_dir'));
            $fcache = new FilesystemTagAwareAdapter('', 0, $cacehDir);
        }
        $entityMapLocal =[];
        $entityFieldMap =[];
        $item = $fcache->getItem(CacheConstants::ENTITY_TYPE_CLASS_MODEL_MAP);
        if (!$item->isHit()) {
            $managerRegistry = $this->container->get('doctrine');
            $queryManager = $managerRegistry->getManagerForClass(ModelMetadata::class);
            $modelRepo = $queryManager->getRepository(ModelMetadata::class);
            $modelMetaDatas = $modelRepo->findAll();
            $systemFixturesFile = file_get_contents('/opt/cyops-api/src/StaticFile/SystemFixtures.json');
            $systemFixture = json_decode($systemFixturesFile);

            foreach ($modelMetaDatas as $modelMetaData) {
                $singular = $modelMetaData->getDescriptions()['singular'];
                $escapedSingular = preg_replace('/[^a-z0-9\s]+/i', '', $singular);
                $escapedSingular = LegacyInflector::classify($escapedSingular);
                $this->utilityService->iterateORMObjectItems($modelMetaData, $escapedSingular, $entityMapLocal, $entityFieldMap);
            }
            foreach ($systemFixture as $modelMetaData) {
                $singular = $modelMetaData->descriptions->singular;
                if ($singular == "Playbook") {
                    $singular = "Workflow";
                } else if ($singular == "Playbook Collection") {
                    $singular = "Workflow Collection";
                } elseif($singular == "Playbook Version")
                {
                    $singular = "Workflow Version";
                }
                $escapedSingular = $this->inflector->classify($singular);
                $this->utilityService->iterateStdClassItems($modelMetaData, $escapedSingular, $entityMapLocal, $entityFieldMap);
            }

            $item->set($entityMapLocal);
            $item->tag(CacheConstants::TAG_MODEL_METADATA);
            $fcache->save($item);

            $entityFieldCache = $fcache->getItem(CacheConstants::ENTITY_FIELD_MODEL_MAP);
            $entityFieldCache->set($entityFieldMap);
            $entityFieldCache->tag(CacheConstants::TAG_MODEL_METADATA);
            $fcache->save($entityFieldCache);
        }
    }
}
