<?php
/* Copyright start
Copyright (C) 2008 - 2021 Fortinet Inc.
All rights reserved.
FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
Copyright end */

namespace App\Service;

use App\Entity\Core\PgFile;
use App\Service\AccessibleTeamIdsProvider;
use Symfony\Component\Filesystem\Filesystem;
use GuzzleHttp\Psr7\Request as GuzzleRequest;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;
use Symfony\Component\Security\Core\Authentication\Token\Storage\TokenStorageInterface;

class UtilityService
{

    const WIDGET_PREVIEW_PATH = '/opt/cyops-ui/widgets_preview/';
    const WIDGET_PATH = '/opt/cyops-ui/widgets/';

    protected $accessibleTeamIdsProvider;

    protected $tokenStorage;

    protected $containers;

    public function __construct(AccessibleTeamIdsProvider $accessibleTeamIdsProvider, TokenStorageInterface $tokenStorage, ContainerInterface $containers)
    {
        $this->accessibleTeamIdsProvider = $accessibleTeamIdsProvider;
        $this->tokenStorage = $tokenStorage;
        $this->containers = $containers;
    }

    public function startsWith($string, $startString)
    {
        $len = strlen($startString);
        return (substr($string, 0, $len) === $startString);
    }

    public function getItemWithWhere($repository, $filters)
    {
        $queryBuilder = $repository->createQueryBuilder('o');

        foreach ($filters as $key => $value) {
            if (gettype($value) == 'array') {
                $queryBuilder->andwhere('o.' . $key . ' in (:' . $key . ')');
                $queryBuilder->setParameter($key, $value);
                continue;
            }
            if (gettype($value) != 'boolean') {
                if ($this->startsWith($value, '/api/3/')) {
                    $explodedValue = explode('/', $value);
                    $value = end($explodedValue);
                }
                $queryBuilder->andwhere('o.' . $key . '= :' . $key);
                $queryBuilder->setParameter($key, $value);
            } else {
                $queryBuilder->andwhere('o.' . $key . '= :value');
                $queryBuilder->setParameter('value', $value);
            }
        }
        return $queryBuilder->getQuery()->execute();
    }

    /**
     * @param $item
     * @param $normalizedTeamData
     * @param $normalizedItemData
     */
    protected function isRecordInList($accessibleRecordTeamUuid, $loggedInUserTeamUuid)
    {
        if (count($accessibleRecordTeamUuid) == 0) {
            // Public record
            return true;
        }
        if (count($loggedInUserTeamUuid) == 0) {
            // User not on any teams
            return false;
        }
        if (count($accessibleRecordTeamUuid) && count($loggedInUserTeamUuid)) {
            // Checking if Record owning teams and logged in user teams have common teams
            if (!count(array_intersect($accessibleRecordTeamUuid, $loggedInUserTeamUuid))) {
                return false;
            }
        }
        return true;
    }

    protected function getRecordTeamUuid($object)
    {
        $record_team = [];
        if (method_exists($object, 'getOwners') && $object->getOwners() != null && $object->getOwners() != []) {
            $record_team = $this->accessibleTeamIdsProvider->getParentDataForTeam($object->getOwners());
        }
        return $record_team;
    }

    protected function getLoggedInUserTeamUuid($object)
    {
        $record_team = [];
        if (method_exists($object, 'getTeams') && $object->getTeams() != null && $object->getTeams() != []) {
            foreach ($object->getTeams() as $team) {
                array_push($record_team, $team->getUuid());
            }
        }
        return $record_team;
    }

    private function getUser()
    {
        return $this->tokenStorage->getToken()->getUser();
    }

    public function ValidateOwnerShip($request, $item, $resource = null)
    {
        if ($request && $item) {
            $accessibleRecordTeamUuid = $this->getRecordTeamUuid($item);
            $loggedInUser = $this->getUser();
            $loggedInUserTeamUuid = $this->getLoggedInUserTeamUuid($loggedInUser);
            if (!count($loggedInUserTeamUuid)) {
                // User with no team should not be allowed to view the resource
                throw new AccessDeniedException();
            }
            // Check if Record is ownable
            $teamOwned = $this->isRecordInList($accessibleRecordTeamUuid, $loggedInUserTeamUuid);
            $userOwned = $this->doesUserOwnRecord($item);
            if (!$teamOwned && !$userOwned) {
                throw new AccessDeniedException();
            }
        }
    }

    protected function doesUserOwnRecord($item)
    {
        if (method_exists($item, 'getUserOwners') 
            && $item->getUserOwners() != null 
            && $item->getUserOwners() != [] 
            && $item->getUserOwners()->contains($this->getUser())) {
            return true;
        }
        return false;
    }

    public function buildWidgetFiles($widget, $path, $entityManager, $operation = 'update')
    {
        $widgetsPath = sprintf("%s/%s", $path, $widget->getName());
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath . "/widgetAssets")) {
            $widgetsFileSystem->remove($widgetsPath . "/widgetAssets");
        }
        if (!$widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->mkdir($widgetsPath, 0755);
            $widgetsFileSystem->mkdir($widgetsPath . "/html", 0755);
            $widgetsFileSystem->mkdir($widgetsPath . "/js", 0755);
        }

        try {
            foreach ($widget->getFiles() as $key => $value) {
                if ($value->getMetadata() && array_key_exists('path', $value->getMetadata())) {
                    $file_explode = explode('/', $value->getFile());
                    $pgFile = $entityManager->find(PgFile::class, end($file_explode));
                    if ($operation == 'create') {
                        $filecontent = base64_decode($pgFile->getFile());
                    } else {
                        $filecontent = base64_decode(stream_get_contents($pgFile->getFile()));
                    }
                    $widgetsFileSystem->dumpFile(sprintf('%s%s/%s', $widgetsPath, $value->getMetadata()['path'], $value->getFilename()), $filecontent);
                }
            }
            $widgetsFileSystem->dumpFile(sprintf('%s/html/view.html', $widgetsPath), $widget->getViewHtml());
            $widgetsFileSystem->dumpFile(sprintf('%s/js/view.controller.js', $widgetsPath), $widget->getViewController());
            $widgetsFileSystem->dumpFile(sprintf('%s/html/edit.html', $widgetsPath), $widget->getEditHtml());
            $widgetsFileSystem->dumpFile(sprintf('%s/js/edit.controller.js', $widgetsPath), $widget->getEditController());
            $infoData['metadata'] = $widget->getMetadata();
            $infoData['name'] = $widget->getName();
            $infoData['title'] = $widget->getTitle();
            $infoData['subTitle'] = $widget->getSubTitle();
            $infoData['version'] = $widget->getVersion();
            if ($widget->getPublishedDate()) {
                $infoData['published_date'] = $widget->getPublishedDate();
            }
            $widgetsFileSystem->dumpFile(sprintf('%s/info.json', $widgetsPath), json_encode($infoData, JSON_PRETTY_PRINT));
        } catch (\Exception $e) {
            throw new HttpException(Response::HTTP_BAD_REQUEST, $e->getMessage(), $e);
        }
        return true;
    }

    public function removeWidgetFiles($widget)
    {
        $widgetsPath = sprintf(self::WIDGET_PATH . "%s", $widget->getName());
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->remove($widgetsPath);
        }
        return true;
    }
    public function removeWidgetPreviewFiles($widgetName)
    {
        $widgetsPath = sprintf(self::WIDGET_PREVIEW_PATH . "%s", $widgetName);
        $widgetsFileSystem = new Filesystem();
        if ($widgetsFileSystem->exists($widgetsPath)) {
            $widgetsFileSystem->remove($widgetsPath);
        }
        return true;
    }

    public function broadcastToHA($commandId, $command)
    {
        $dasUri = $this->containers->getParameter('das.uri');
        $dasClient = $this->containers->get("app.proxy.client.das");
        $objectData = array(
            "commandId" => $commandId,
            "command" => $command,
        );
        $headers = array(
            "Content-type" => "application/json",
        );
        $dasRequests = new GuzzleRequest('post', $dasUri . '/clustercommand/', $headers, json_encode($objectData));
        try {
            $dasResponse = $dasClient->send($dasRequests);
            $resCode = $dasResponse->getStatusCode();
            if ($resCode > 299) {
                return 0;
            }
        } catch (\Exception $error) {
            error_log($error->getMessage());
        }
        return true;
    }

    public function iterateORMObjectItems($item, $escapedSingular, &$entityMapLocal, &$entityFieldMap)
    {
        $htmlEscapeFields = array();
        $fileFields = array();
        $writeableFields = array();
        $classPath = $this->containers->getParameter("type_map")[$escapedSingular] ?? 'App\\Entity\\Custom\\' . $escapedSingular;
        $type = $item->getType();
        $module = $item->getModule();
        $entityMapLocal[$type] = [$classPath, $module];
        $attributes = $item->getAttributes();
        foreach ($attributes as $attribute) {
            if ($attribute->getFormType() == "file") {
                $fileFields[] = $attribute->getName();
            }
            if ($attribute->getHtmlEscape() == true) {
                $htmlEscapeFields[] = $attribute->getName();
            }
            if ($attribute->getWriteable() == false) {
                $writeableFields[] = $attribute->getName();
            }
        }
        $entityFieldMap[$classPath] = ["fileFields" => $fileFields, "htmlEscapeFields" => $htmlEscapeFields, "writeableFields" => $writeableFields];
    }

    public function iterateStdClassItems($item, $escapedSingular, &$entityMapLocal, &$entityFieldMap)
    {
        $htmlEscapeFields = array();
        $fileFields = array();
        $writeableFields = array();
        $classPath = $this->containers->getParameter("type_map")[$escapedSingular] ?? 'App\\Entity\\Custom\\' . $escapedSingular;
        $type = $item->type;
        $module = $item->module;
        $entityMapLocal[$type] = [$classPath, $module];
        $attributes = $item->attributes;
        foreach ($attributes as $attribute) {
            if ($attribute->formType == "file") {
                $fileFields[] = $attribute->name;
            }
            if ($attribute->htmlEscape == true) {
                $htmlEscapeFields[] = $attribute->name;
            }
            if ($attribute->writeable == false) {
                $writeableFields[] = $attribute->name;
            }
        }
        $entityFieldMap[$classPath] = ["fileFields" => $fileFields, "htmlEscapeFields" => $htmlEscapeFields, "writeableFields" => $writeableFields];
    }

    public static function identifyChangeFields($requestBody)
    {
        $keys = ["fsr_primary", "password"];
        foreach ($requestBody as $key => $value) {
            $keys[] = $key;
            if ($key == '__link') {
                foreach ($requestBody->{'__link'} as $innerKey => $innerValue) {
                    $keys[] = $innerKey;
                }
            }
            if ($key == '__unlink') {
                foreach ($requestBody->{'__unlink'} as $innerKey => $innerValue) {
                    $keys[] = $innerKey;
                }
            }
        }
        return $keys;
    }
}
